{
 "cells": [
  {
   "cell_type": "code",
   "execution_count": 1,
   "id": "4d9853ed",
   "metadata": {},
   "outputs": [],
   "source": [
    "#!/usr/bin/env python\n",
    "# encoding: utf-8\n",
    "'''\n",
    "@license: (C) Copyright 2021, Hey.\n",
    "@author: Hey\n",
    "@email: sanyuan.hy@alibaba-inc.com\n",
    "@tel: 137****6540\n",
    "@datetime: 2023/4/9 13:46\n",
    "@project: LucaPCycle\n",
    "@file: file operator\n",
    "@desc: file operator\n",
    "'''\n",
    "import os\n",
    "import csv, sys\n",
    "import io, textwrap, itertools\n",
    "from Bio import SeqIO\n",
    "from Bio.Seq import Seq\n",
    "from Bio.SeqRecord import SeqRecord\n",
    "csv.field_size_limit(sys.maxsize)\n",
    "\n",
    "common_amino_acid_set = {'R', 'X', 'S', 'G', 'W', 'I', 'Q', 'A', 'T', 'V', 'K', 'Y', 'C', 'N', 'L', 'F', 'D', 'M', 'P', 'H', 'E'}\n",
    "\n",
    "\n",
    "def clean_seq(protein_id, seq):\n",
    "    seq = seq.upper()\n",
    "    new_seq = \"\"\n",
    "    has_invalid_char = False\n",
    "    invalid_char_set = set()\n",
    "    for ch in seq:\n",
    "        if 'A' <= ch <= 'Z' and ch not in ['J']:\n",
    "            new_seq += ch\n",
    "        else:\n",
    "            invalid_char_set.add(ch)\n",
    "            has_invalid_char = True\n",
    "    if has_invalid_char:\n",
    "        print(\"id: %s. Seq: %s\" % (protein_id, seq))\n",
    "        print(\"invalid char set:\", invalid_char_set)\n",
    "    return new_seq\n",
    "\n",
    "\n",
    "def file_reader(filename, header=True, header_filter=True):\n",
    "    if filename.endswith(\".fa\") or filename.endswith(\".fas\") or filename.endswith(\".fasta\"):\n",
    "        return fasta_reader(filename)\n",
    "    elif filename.endswith(\".csv\"):\n",
    "        return csv_reader(filename, header=True, header_filter=True)\n",
    "    elif filename.endswith(\".tsv\"):\n",
    "        return tsv_reader(filename, header=True, header_filter=True)\n",
    "    else:\n",
    "        return txt_reader(filename, header=header, header_filter=header_filter)\n",
    "\n",
    "\n",
    "def txt_reader(handle, header=True, header_filter=True):\n",
    "    '''\n",
    "    csv 读取器，适合大文件\n",
    "    :param handle:\n",
    "    :param header:\n",
    "    :param header_filter: 返回结果是否去掉头\n",
    "    :return:\n",
    "    '''\n",
    "    handle = handle if isinstance(handle, io.TextIOWrapper) else open(handle, 'r')\n",
    "    try:\n",
    "        cnt = 0\n",
    "        for line in handle:\n",
    "            cnt += 1\n",
    "            if header and header_filter and cnt == 1:\n",
    "                continue\n",
    "            yield line.strip()\n",
    "    except Exception as e:\n",
    "        raise StopIteration\n",
    "    finally:\n",
    "        if not handle.closed:\n",
    "            handle.close()\n",
    "\n",
    "\n",
    "def tsv_reader(handle, header=True, header_filter=True):\n",
    "    '''\n",
    "    csv 读取器，适合大文件\n",
    "    :param handle:\n",
    "    :param header:\n",
    "    :param header_filter: 返回结果是否去掉头\n",
    "    :return:\n",
    "    '''\n",
    "    handle = handle if isinstance(handle, io.TextIOWrapper) else open(handle, 'r')\n",
    "    try:\n",
    "        reader = csv.reader(handle, delimiter=\"\\t\")\n",
    "        cnt = 0\n",
    "        for row in reader:\n",
    "            cnt += 1\n",
    "            if header and header_filter and cnt == 1:\n",
    "                continue\n",
    "            yield row\n",
    "    except Exception as e:\n",
    "        raise StopIteration\n",
    "    finally:\n",
    "        if not handle.closed:\n",
    "            handle.close()\n",
    "\n",
    "\n",
    "def csv_reader(handle, header=True, header_filter=True):\n",
    "    '''\n",
    "    csv 读取器，适合大文件\n",
    "    :param handle:\n",
    "    :param header:\n",
    "    :param header_filter: 返回结果是否去掉头\n",
    "    :return:\n",
    "    '''\n",
    "    handle = handle if isinstance(handle, io.TextIOWrapper) else open(handle, 'r')\n",
    "    try:\n",
    "        # data = csv.reader((line.replace('\\0','') for line in data_initial), delimiter=\",\")\n",
    "        # reader = csv.reader(handle)\n",
    "        reader = csv.reader((line.replace('\\0', '') for line in handle))\n",
    "        cnt = 0\n",
    "        for row in reader:\n",
    "            cnt += 1\n",
    "            if header and header_filter and cnt == 1:\n",
    "                continue\n",
    "            yield row\n",
    "    except Exception as e:\n",
    "        raise StopIteration\n",
    "    finally:\n",
    "        if not handle.closed:\n",
    "            handle.close()\n",
    "\n",
    "\n",
    "def txt_writer(dataset, handle, header=None):\n",
    "    '''\n",
    "    txt 写\n",
    "    :param dataset: 数据\n",
    "    :param handle: 文件\n",
    "    :param header: 头\n",
    "    :return:\n",
    "    '''\n",
    "    with open(handle, \"w\") as wfp:\n",
    "        if header:\n",
    "            if isinstance(header, list):\n",
    "                wfp.write(\",\".join(header) + \"\\n\")\n",
    "            else:\n",
    "                wfp.write(header + \"\\n\")\n",
    "        for row in dataset:\n",
    "            wfp.write(str(row) + \"\\n\")\n",
    "\n",
    "\n",
    "def csv_writer(dataset, handle, header):\n",
    "    '''\n",
    "    csv 写，适合大文件\n",
    "    :param dataset: 数据\n",
    "    :param handle: 文件\n",
    "    :param header: 头\n",
    "    :return:\n",
    "    '''\n",
    "    handle = handle if isinstance(handle, io.TextIOWrapper) else open(handle, 'w')\n",
    "    try:\n",
    "        writer = csv.writer(handle)\n",
    "        if header:\n",
    "            writer.writerow(header)\n",
    "        for row in dataset:\n",
    "            writer.writerow(row)\n",
    "    except Exception as e:\n",
    "        raise e\n",
    "    finally:\n",
    "        if not handle.closed:\n",
    "            handle.close()\n",
    "\n",
    "\n",
    "def tsv_writer(dataset, handle, header):\n",
    "    '''\n",
    "    tsv 写，适合大文件\n",
    "    :param dataset: 数据\n",
    "    :param handle: 文件\n",
    "    :param header: 头\n",
    "    :return:\n",
    "    '''\n",
    "    handle = handle if isinstance(handle, io.TextIOWrapper) else open(handle, 'w')\n",
    "    try:\n",
    "        writer = csv.writer(handle, delimiter=\"\\t\")\n",
    "        if header:\n",
    "            writer.writerow(header)\n",
    "        for row in dataset:\n",
    "            writer.writerow(row)\n",
    "    except Exception as e:\n",
    "        raise e\n",
    "    finally:\n",
    "        if not handle.closed:\n",
    "            handle.close()\n",
    "\n",
    "def fasta_reader(handle, width=None):\n",
    "    \"\"\"\n",
    "    Reads a FASTA file, yielding header, sequence pairs for each sequence recovered 适合大文件\n",
    "    args:\n",
    "        :handle (str, pathliob.Path, or file pointer) - fasta to read from\n",
    "        :width (int or None) - formats the sequence to have max `width` character per line.\n",
    "                               If <= 0, processed as None. If None, there is no max width.\n",
    "    yields:\n",
    "        :(header, sequence) tuples\n",
    "    returns:\n",
    "        :None\n",
    "    \"\"\"\n",
    "    FASTA_STOP_CODON = \"*\"\n",
    "\n",
    "    handle = handle if isinstance(handle, io.TextIOWrapper) else open(handle, 'r')\n",
    "    width = width if isinstance(width, int) and width > 0 else None\n",
    "    try:\n",
    "        header = None\n",
    "        for is_header, group in itertools.groupby(handle, lambda line: line.startswith(\">\")):\n",
    "            if is_header:\n",
    "                header = group.__next__().strip()\n",
    "            else:\n",
    "                seq = ''.join(line.strip() for line in group).strip().rstrip(FASTA_STOP_CODON)\n",
    "                if width is not None:\n",
    "                    seq = textwrap.fill(seq, width)\n",
    "                yield header, seq\n",
    "    except Exception as e:\n",
    "        raise StopIteration\n",
    "    finally:\n",
    "        if not handle.closed:\n",
    "            handle.close()\n",
    "\n",
    "\n",
    "def write_fasta(filepath, sequences):\n",
    "    '''\n",
    "    write fasta file\n",
    "    :param filepath: savepath\n",
    "    :param sequences: fasta sequence(each item: [id, seq])\n",
    "    :return:\n",
    "    '''\n",
    "\n",
    "    if sequences:\n",
    "        with open(filepath, \"w\") as output_handle:\n",
    "            if len(sequences[0]) > 1 and isinstance(sequences[0][0], str):\n",
    "                for row in sequences:\n",
    "                    protein_id = row[0]\n",
    "                    seq = row[1]\n",
    "                    sequence = SeqRecord(Seq(seq, None), id=protein_id[1:] if protein_id and protein_id[0] == \">\" else protein_id, description=\"\")\n",
    "                    SeqIO.write(sequence, output_handle, \"fasta\")\n",
    "            else:\n",
    "                for sequence in sequences:\n",
    "                    SeqIO.write(sequence, output_handle, \"fasta\")\n"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 2,
   "id": "a4e06884",
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "1481237\n"
     ]
    }
   ],
   "source": [
    "# lucapcycle v2\n",
    "unchecked_lucapcycle_v2 = \"./lucapcycle_v2_init.csv\"\n",
    "unchecked_lucapcycle_v2_ids_labels = []\n",
    "for row in csv_reader(unchecked_lucapcycle_v2, header=True, header_filter=True):\n",
    "    seq_id = row[0]\n",
    "    if seq_id[0] == \">\":\n",
    "        seq_id = seq_id[1:]\n",
    "    unchecked_lucapcycle_v2_ids_labels.append([seq_id, row[3]])\n",
    "csv_writer(unchecked_lucapcycle_v2_ids_labels, \"./lucapcycle_v2_init.ids.labels\", header=[\"seq_id\", \"label\"])\n",
    "print(len(unchecked_lucapcycle_v2_ids_labels))"
   ]
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python 3 (ipykernel)",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.9.13"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 5
}
